@@ -33,7 +33,7 @@ fn derive_censoring() {
 //- proc_macros: derive_identity
 #[attr1]
 #[derive(Foo)]
-#[derive(proc_macros::derive_identity)]
+#[derive(proc_macros::DeriveIdentity)]
 #[derive(Bar)]
 #[attr2]
 struct S;
@@ -41,7 +41,7 @@ struct S;
         expect![[r##"
 #[attr1]
 #[derive(Foo)]
-#[derive(proc_macros::derive_identity)]
+#[derive(proc_macros::DeriveIdentity)]
 #[derive(Bar)]
 #[attr2]
 struct S;
